@@ -3,7 +3,7 @@
 >>> from lazysequence import lazysequence
 >>>
 >>> def load_records():
-...     yield from [1, 2, 3, 4, 5, 6]  # pretend each iteration is expensive
+...     return range(10)  # let's pretend this is expensive
 ...
 >>> records = lazysequence(load_records())
 >>> if not records:
@@ -12,19 +12,23 @@
 >>> first, second = records[:2]
 >>>
 >>> print("The first record is", first)
-The first record is 1
+The first record is 0
 >>> print("The second record is", second)
-The second record is 2
+The second record is 1
 >>>
 >>> for record in records.release():  # do not cache all records in memory
 ...     print("record", record)
 ...
+record 0
 record 1
 record 2
 record 3
 record 4
 record 5
 record 6
+record 7
+record 8
+record 9
 """
 from __future__ import annotations
 
@@ -76,7 +80,8 @@ class lazysequence(Sequence[_T_co]):  # noqa: N801
     def release(self) -> Iterator[_T_co]:
         """Iterate over the sequence without caching additional items.
 
-        The sequence should no longer be used after calling this function.
+        The sequence should no longer be used after calling this function. The
+        same applies to slices of the sequence obtained by using ``s[i:j]``.
 
         Yields:
             The items in the sequence.
@@ -101,7 +106,7 @@ class lazysequence(Sequence[_T_co]):  # noqa: N801
     def __getitem__(self, indices: slice) -> lazysequence[_T_co]:
         """Return a slice of the sequence."""  # noqa: D418
 
-    def __getitem__(
+    def __getitem__(  # noqa: C901
         self, index: Union[int, slice]
     ) -> Union[_T_co, lazysequence[_T_co]]:
         """Return the item at the given index."""
@@ -109,19 +114,41 @@ class lazysequence(Sequence[_T_co]):  # noqa: N801
             start, stop, step = index.start, index.stop, index.step
 
             if step is not None and step < 0:
-                return lazysequence(reversed(self[start:stop:-step]))
+                size = len(self)
+                step = -step
+
+                if start is None:
+                    start = 0
+                else:
+                    start = (size - 1) - start
+
+                start = max(0, start)
+
+                if stop is None:
+                    stop = size
+                else:
+                    stop = (size - 1) - stop
+
+                stop = max(0, stop)
+
+                return lazysequence(islice(reversed(self._cache), start, stop, step))
 
             if start is not None and start < 0:
                 start += len(self)
+                start = max(0, start)
 
             if stop is not None and stop < 0:
                 stop += len(self)
+                stop = max(0, stop)
 
             return lazysequence(islice(self, start, stop, step))
 
         if index < 0:
             index += len(self)
 
+        if index < 0:
+            raise IndexError("lazysequence index out of range")
+
         try:
             return self._cache[index]
         except IndexError:
